package com.ruoyi.project.pssr.service;

import java.util.List;
import com.ruoyi.project.pssr.domain.TPssrSafetyFlamearrester;

/**
 * 安全设施-阻火器Service接口
 * 
 * @author ssy
 * @date 2024-09-18
 */
public interface ITPssrSafetyFlamearresterService 
{
    /**
     * 查询安全设施-阻火器
     * 
     * @param id 安全设施-阻火器ID
     * @return 安全设施-阻火器
     */
    public TPssrSafetyFlamearrester selectTPssrSafetyFlamearresterById(Long id);

    /**
     * 查询安全设施-阻火器列表
     * 
     * @param tPssrSafetyFlamearrester 安全设施-阻火器
     * @return 安全设施-阻火器集合
     */
    public List<TPssrSafetyFlamearrester> selectTPssrSafetyFlamearresterList(TPssrSafetyFlamearrester tPssrSafetyFlamearrester);

    /**
     * 新增安全设施-阻火器
     * 
     * @param tPssrSafetyFlamearrester 安全设施-阻火器
     * @return 结果
     */
    public int insertTPssrSafetyFlamearrester(TPssrSafetyFlamearrester tPssrSafetyFlamearrester);

    /**
     * 修改安全设施-阻火器
     * 
     * @param tPssrSafetyFlamearrester 安全设施-阻火器
     * @return 结果
     */
    public int updateTPssrSafetyFlamearrester(TPssrSafetyFlamearrester tPssrSafetyFlamearrester);
    public int updateAllBySubId(TPssrSafetyFlamearrester tPssrSafetyFlamearrester);

    /**
     * 批量删除安全设施-阻火器
     * 
     * @param ids 需要删除的安全设施-阻火器ID
     * @return 结果
     */
    public int deleteTPssrSafetyFlamearresterByIds(Long[] ids);

    /**
     * 删除安全设施-阻火器信息
     * 
     * @param id 安全设施-阻火器ID
     * @return 结果
     */
    public int deleteTPssrSafetyFlamearresterById(Long id);
}
