package com.ruoyi.project.pssr.controller;

import com.alibaba.fastjson.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.ExcelUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.pssr.domain.TPssrApprove;
import com.ruoyi.project.pssr.domain.TPssrFile;
import com.ruoyi.project.pssr.domain.TPssrSafetyFlamearrester;
import com.ruoyi.project.pssr.mapper.TPssrSafetyFlamearresterMapper;
import com.ruoyi.project.pssr.service.ITPssrApproveService;
import com.ruoyi.project.pssr.service.ITPssrFileService;
import com.ruoyi.project.pssr.service.ITPssrSafetyFlamearresterService;
import com.ruoyi.project.pssr.service.ITPssrTurndownService;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysDeptService;
import com.ruoyi.project.system.service.ISysUserService;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngines;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * 安全设施-阻火器Controller
 *
 * @author ssy
 * @date 2024-09-18
 */
@RestController
@RequestMapping("/pssr/safetyFlamearrester")
public class TPssrSafetyFlamearresterController extends BaseController {
    @Autowired
    private ITPssrFileService tPssrFileService;

    @Autowired
    private ITPssrTurndownService tPssrTurndownService;

    @Autowired
    private ITPssrApproveService tPssrApproveService;

    @Autowired
    private ITPssrSafetyFlamearresterService tPssrSafetyFlamearresterService;

    @Autowired
    private ISysUserService sysUserService;
    private String forShort = "aqss-f";

    @Autowired
    private ISysDeptService iSysDeptService;

    @Autowired
    private ISysUserService userService;
    @Resource
    private TPssrSafetyFlamearresterMapper tPssrSafetyFlamearresterMapper;
    /**
     * 批量导入
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:add')")
    @PostMapping("/importData")
    public AjaxResult importInterlockData(MultipartFile file, Long subId) throws IOException
    {
        //获取操作人员ID
        Long userId = getUserId();
        //报错行数统计
        List<Integer> failRow =new ArrayList<Integer>();
        Workbook workbook = ExcelUtils.getWorkBook(file);
        Sheet sheet = workbook.getSheetAt(0);
        List<TPssrSafetyFlamearrester> list = new ArrayList<TPssrSafetyFlamearrester>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //部门查询
        List<SysDept> dept = iSysDeptService.selectDeptList(new SysDept());
        int rowNum = sheet.getPhysicalNumberOfRows();
        int failNumber = 0;
        for (int i = 2; i <= rowNum; i++) {
            try {
                logger.info("读取行数：" + i);
                Row row = sheet.getRow(i);
                int cellNum = row.getLastCellNum();
                TPssrSafetyFlamearrester entity = new TPssrSafetyFlamearrester();
                entity.setDeptId(userService.selectUserById(getUserId()).getDeptId());
                entity.setSubId(subId);
                entity.setApproveStatus(0L);
                for (int j = 0; j < cellNum; j++) {
                    Cell cell = row.getCell(j);
                    String cellValue = ExcelUtils.getCellValue(cell);
                    logger.info("cellValue:" + cellValue);
                    if (j == 0) {
                        entity.setUnit(cellValue);
                    } else if (j == 1) {
                        entity.setDevNo(cellValue);
                    } else if (j == 2) {
                        entity.setInstallLocation(cellValue);
                    } else if (j == 3) {
                        entity.setConfirmer1(cellValue);
                    } else if (j == 4) {
                        entity.setConfirmer2(cellValue);
                    }
                }
                entity.setCreaterCode(userId.toString());
                logger.info("entity:" + entity);
                list.add(entity);
            }catch (Exception e){
                failNumber++;
                failRow.add(i+1);
            }
        }
        int successNumber = 0;
        int failNum = 0;
        for (TPssrSafetyFlamearrester t : list
        ) {
            failNum++;
            try {
                this.add(t);
                successNumber++;
            }catch (Exception e){
                failNumber++;
                logger.info("e:" + e);
                failRow.add(failNum+1);
            }
        }
        logger.info("list:" + JSON.toJSONString(list));
        logger.info("successNumber:" +String.valueOf(successNumber));
        logger.info("failNumber:" +String.valueOf(failNumber));
        logger.info("failRow:" +String.valueOf(failRow));
        return AjaxResult.success("导入成功行数：" + String.valueOf(successNumber));
    }

    /**
     * 查询安全设施-阻火器列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:list')")
    @GetMapping("/list")
    public TableDataInfo list(TPssrSafetyFlamearrester tPssrSafetyFlamearrester) {
        try {
            TPssrApprove approve = tPssrApproveService.selectTPssrApproveBySubId(tPssrSafetyFlamearrester.getSubId());
            if (approve != null) {
                ProcessEngine processEngine = ProcessEngines.getDefaultProcessEngine();
                TaskService taskService = processEngine.getTaskService();
                Task task = processEngine.getTaskService()//获取任务service
                        .createTaskQuery()//创建查询对象
                        .taskAssignee(getUserId().toString())
                        .processInstanceId(approve.getProcessId()).singleResult();
                if (task != null) {
                    if (task.getName().equals("确认人1")){
                        tPssrSafetyFlamearrester.setConfirmer1(getUserId().toString());
                    }else if (task.getName().equals("确认人2")){
                       tPssrSafetyFlamearrester.setConfirmer2(getUserId().toString());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("待办确认人查询报错：{}",e.getMessage());
        }
        startPage();
        List<TPssrSafetyFlamearrester> list = tPssrSafetyFlamearresterService.selectTPssrSafetyFlamearresterList(tPssrSafetyFlamearrester);
        list.forEach(item -> {
            
            List<TPssrFile> tPssrFiles = tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort);
            item.setFileList(tPssrFiles);
            item.setFileNum((long) tPssrFiles.size());
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "aqss-f"));
        });
        return getDataTable(list);
    }

    /**
     * 导出安全设施-阻火器列表
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:export')")
    @Log(title = "安全设施-阻火器", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TPssrSafetyFlamearrester tPssrSafetyFlamearrester) {
        List<TPssrSafetyFlamearrester> list = tPssrSafetyFlamearresterService.selectTPssrSafetyFlamearresterList(tPssrSafetyFlamearrester);
        if (list.size() != 0) {
            return AjaxResult.success(exportTmpl(list));
        } else {
            return AjaxResult.error("暂无可导出数据");
        }
    }

    public String exportTmpl(List<TPssrSafetyFlamearrester> list) {
        OutputStream out = null;
        String filename = null;
        try {
            String tempUrl = "static/word/pssr/aqsszhq.xlsx"; // 模板文件
            InputStream is = null;
            is = Thread.currentThread().getContextClassLoader().getResourceAsStream(tempUrl);
            XSSFWorkbook wb = null;
            wb = new XSSFWorkbook(is);
            XSSFSheet sheet = wb.getSheetAt(0);

            //填充数据
            int rowIndex = 4;
            int num = 1;

            Row originalRow = sheet.getRow(4);
            Cell originalcell = originalRow.getCell(0);
            // 获取单元格样式
            CellStyle originalStyle = originalcell.getCellStyle();

            for (TPssrSafetyFlamearrester t : list) {
                Row row = sheet.createRow(rowIndex);
                row.setHeight((short) 800);
                row.createCell(0).setCellValue(num);
                row.createCell(1).setCellValue(t.getUnit());
                row.createCell(2).setCellValue(t.getDevNo());
                row.createCell(3).setCellValue(t.getInstallLocation());
                row.createCell(4).setCellValue(t.getUniformFlow());
                row.createCell(5).setCellValue(t.getCorrosion());
                row.createCell(6).setCellValue(t.getPutUse());
                row.createCell(7);
                row.createCell(8);
                try {
                    SysUser sysUser1 = sysUserService.selectUserById(Long.valueOf(t.getConfirmer1()));
                    SysUser sysUser2 = sysUserService.selectUserById(Long.valueOf(t.getConfirmer2()));
                    String confirm1 = sysUser1.getSignUrl();
                    String confirm2 = sysUser2.getSignUrl();
                    ExcelUtils.insertPicture(wb, sheet, confirm1, row.getRowNum(), 7, 1, 1);
                    ExcelUtils.insertPicture(wb, sheet, confirm2, row.getRowNum(), 8, 1, 1);
                } catch (Exception e) {

                }
                row.createCell(9).setCellValue(DateUtils.dateTime(t.getConfirmationTime()));
                row.createCell(10).setCellValue(t.getRemarks());

                //渲染样式
                for (int i = 0; i < 11; i++) {
                    row.getCell(i).setCellStyle(originalStyle);
                }
                num++;
                rowIndex++;
            }
            filename = "PSSR_14_安全设施_阻火器"  + ".xlsx";
            out = new FileOutputStream(ExcelUtil.getAbsoluteFile(filename));
            wb.write(out);
            wb.close();

        } catch (IOException e) {
            e.printStackTrace();
        }
        return filename;
    }

    /**
     * 获取安全设施-阻火器详细信息
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TPssrSafetyFlamearrester item = tPssrSafetyFlamearresterService.selectTPssrSafetyFlamearresterById(id);
        
            List<TPssrFile> tPssrFiles = tPssrFileService.selectTPssrFileListByItem(item.getSubId(), item.getId(), forShort);
            item.setFileList(tPssrFiles);
            item.setFileNum((long) tPssrFiles.size());
            if (item.getApproveStatus() != 2)
                item.setReason(tPssrTurndownService.selectTPssrTurndownByItem(item.getSubId(), item.getId(), "aqss-f"));
        return AjaxResult.success(item);
    }

    /**
     * 新增安全设施-阻火器
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:add')")
    @Log(title = "安全设施-阻火器", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TPssrSafetyFlamearrester tPssrSafetyFlamearrester) {
        if (StringUtils.isNotEmpty(tPssrSafetyFlamearrester.getConfirmer1())&&tPssrSafetyFlamearrester.getConfirmer1().equals(tPssrSafetyFlamearrester.getConfirmer2())) {
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        tPssrSafetyFlamearrester.setApproveStatus(0L);
        return toAjax(tPssrSafetyFlamearresterService.insertTPssrSafetyFlamearrester(tPssrSafetyFlamearrester));
    }

    //导入时判断更新或新增
    private int insertOrUpdate(TPssrSafetyFlamearrester item) {
        TPssrSafetyFlamearrester entity = new TPssrSafetyFlamearrester();
        entity.setSubId(item.getSubId());
        entity.setDevNo(item.getDevNo());
        List<TPssrSafetyFlamearrester> list = tPssrSafetyFlamearresterService.selectTPssrSafetyFlamearresterList(entity);
        if (CollectionUtils.isNotEmpty(list)) {
            item.setId(list.get(0).getId());
            return tPssrSafetyFlamearresterService.updateTPssrSafetyFlamearrester(item);
        } else {
            return tPssrSafetyFlamearresterService.insertTPssrSafetyFlamearrester(item);
        }
    }

    /**
     * 修改安全设施-阻火器
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:edit')")
    @Log(title = "安全设施-阻火器", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TPssrSafetyFlamearrester tPssrSafetyFlamearrester) {
        TPssrSafetyFlamearrester entity = tPssrSafetyFlamearresterService.selectTPssrSafetyFlamearresterById(tPssrSafetyFlamearrester.getId());
        if (entity.getApproveStatus() != 1 && entity.getApproveStatus() != 0) {
            return  AjaxResult.error("当前状态不可修改！");
        }
        if (tPssrSafetyFlamearrester.getConfirmer1().equals(tPssrSafetyFlamearrester.getConfirmer2())){
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        tPssrFileService.updateFileRelevance(tPssrSafetyFlamearrester.getFileIds(), "aqss-f", tPssrSafetyFlamearrester.getId(), tPssrSafetyFlamearrester.getSubId());
        return toAjax(tPssrSafetyFlamearresterService.updateTPssrSafetyFlamearrester(tPssrSafetyFlamearrester));
    }

    /**
     * 修改安全设施-阻火器
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:edit')")
    @Log(title = "安全设施-阻火器", businessType = BusinessType.UPDATE)
    @PutMapping("/editBatch")
    public AjaxResult editb(@RequestBody TPssrSafetyFlamearrester tPssrSafetyFlamearrester) {
        if (tPssrSafetyFlamearrester.getConfirmer1().equals(tPssrSafetyFlamearrester.getConfirmer2())){
            return AjaxResult.error("确认人不能为同一人，请重新选择！");
        }
        return toAjax(tPssrSafetyFlamearresterMapper.updateTPssrSafetyFlamearresterByIds(tPssrSafetyFlamearrester));
    }

    /**
     * 删除安全设施-阻火器
     */
    @PreAuthorize("@ss.hasPermi('pssr:safetyFlamearrester:remove')")
    @Log(title = "安全设施-阻火器", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tPssrSafetyFlamearresterService.deleteTPssrSafetyFlamearresterByIds(ids));
    }
}
